﻿// XML soundboard by Jeremy Markowitz
// jeremymarkowitz@yahoo.com
// deltadada@deltadada.com

package soundBoard {
	import flash.display.*;
	import flash.events.*;
	import flash.net.*;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.net.URLRequest;
	import flash.errors.IOError;
	import flash.events.IOErrorEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.utils.Timer;
	import flashx.textLayout.elements.ParagraphElement;
	import com.greensock.TweenLite;
	import com.greensock.easing.Elastic;
	import soundBoard.CustomSoundButton;
	import soundBoard.RoundBtn;

	/**
	 * @author JeremyMarkowitz
	 * 
	 * Soundboard.as is the main view of the soundboard.  It creates a SoundboardModel to hold various states and variables.
	 * With the model instantiated, all the buttons are placed, event listeners are created, and initial states are set.
	 * 
	 */
	
	public class Soundboard extends Sprite {
		static public const FEVER_ON_TEXT:String = "Fever Dream Mode";
		static public const FEVER_OFF_TEXT:String = "Stop Fever Dream";
		static public const CUSTOM_ADD_TEXT:String = "Drag to Custom Page";
		static public const CUSTOM_DELETE_TEXT:String = "Delete from Custom";
		static public const CUSTOM_OVER_TEXT:String = "Drag Button Here";
		static public const CLEAR_CUSTOM_TEXT:String = "Clear All";
		static public const PLAY_SEQUENCE_TEXT:String = "Play Sequence";
		static public const NO_SOUNDS_TEXT:String = 'Drag sounds to "' + CUSTOM_ADD_TEXT + '" button';
		static public const USER_PAGE_FULL_TEXT:String = 'Sound not added -- User Page full!';
		static public const TIMER_WAIT:uint = 22;
		static public const ALERT_BG:uint = 0xD9D9D9;
		
		private var xmlLoader:XmlLoader;
		private var datasource:String;

		private var sndC:SoundChannel;
		private var pageGutter:Number;
		private var pagesHolder:Sprite = new Sprite();
		private var tfTitle:TextField;
		private var tfAlert:TextField;
		private var btnFeverDream:RoundBtn;
		private var btnCustom:DragToButton;
		private var btnCustomClear:RoundBtn;
		private var btnPlaySequence:RoundBtn;

		private var soundBtnHoldTimer:Timer;

		public var model:IModel;

		public function Soundboard(datasource:String) {
			model = new SoundboardModel(stage);
			
			model.btnActionState = new SoundboardButtonState(model);
			model.addEventListener(SoundboardModel.SWAP, this.swapUpdate);
			model.addEventListener(SoundboardModel.GHOST_NULL, this.removeGhostButton);
			model.addEventListener(SoundboardModel.PLAY_SELECTED_SOUND, this.playSoundUpdate);
			model.addEventListener(SoundboardModel.CUSTOM_ADD, this.customAddButtonUpdate);
			model.addEventListener(SoundboardModel.CUSTOM_DELETE, this.customDeleteButtonUpdate);
			model.addEventListener(SoundboardModel.SOUNDS_CHANGE, this.initButton);

			getAppXML(datasource);
		} //
		


		private function getAppXML(xmlPath:String):void {
			xmlLoader = new XmlLoader();
			xmlLoader.addEventListener(XmlLoader.XML_LOADED, init);
			xmlLoader.loadXML(xmlPath);
		}

		private function init(e:Event):void {
			//var bgHolder:Sprite = new Sprite();
			//bgHolder.addChild(Bitmap(xmlLoader.getBackgroundImg().content));
			model.bgImg = Bitmap(xmlLoader.getBackgroundImg().content);
			
			addChild(model.bgImg);
			model.bgFeverImg = new Bitmap(model.bgImg.bitmapData.clone()); 
			model.bgFeverImg.alpha = 0;
			addChild(model.bgFeverImg);
			
			if (!xmlLoader.getAppXML()){
				trace("Soundboard: XML Load FAIL");
				return;
			} else {
				trace("Soundboard: xml loaded\n");
				model.dataXML = XML(xmlLoader.getAppXML());
				sndC = new SoundChannel();

				makeSampleButtons();
				model.soundChannelState = new SoundboardSoundState(model);
				makeUIButtons();

			}
		} //

		private function makeUIButtons():void {
			// make title
			tfTitle = new TextField();
			tfTitle.selectable = false;
			tfTitle.autoSize = TextFieldAutoSize.LEFT;
			tfTitle.background = false;

			var txtFmt:TextFormat = new TextFormat();
			txtFmt.font = "Verdana";
			txtFmt.size = 16;
			txtFmt.bold = true;
			txtFmt.color = 0xFFFFFF;
			//txtFmt.letterSpacing = 1;
			tfTitle.defaultTextFormat = txtFmt;
			tfTitle.setTextFormat(txtFmt);
			tfTitle.text = model.appTitleText;
			addChild(tfTitle);
			tfTitle.x = model.aPages[0].x;
			tfTitle.y = 6;

			makePageButtons();
			model.aPgBtns[0].setActive(false);

			// put pages below title
			for each (var page:Sprite in model.aPages){
				page.y = tfTitle.y + tfTitle.height + model.gutter;
			}
			
			//Fever dream button
			btnFeverDream = new RoundBtn(FEVER_ON_TEXT, 12);
			btnFeverDream.x = 14;
			btnFeverDream.y = model.aPages[0].y + model.aPages[0].height + (model.gutter * 2);
			btnFeverDream.addEventListener(MouseEvent.CLICK, feverBtnClickHandler);
			addChild(btnFeverDream);
			
			// Drag to Custom button
			btnCustom = new DragToButton(CUSTOM_ADD_TEXT, 12);
			btnCustom.y = btnFeverDream.y;
			btnCustom.x = model.aPages[0].x + model.aPages[0].width - btnCustom.width;
			btnCustom.addEventListener(MouseEvent.MOUSE_OVER, customBtnOverHandler);
			btnCustom.addEventListener(MouseEvent.MOUSE_OUT, customBtnOutHandler);
			addChild(btnCustom);
			
			// custom clean button
			btnCustomClear = new RoundBtn(CLEAR_CUSTOM_TEXT, 12);
			btnCustomClear.y = stage.stageHeight+2;
			btnCustomClear.x = btnCustom.x -(model.gutter * 2) - btnCustomClear.width;
			btnCustomClear.addEventListener(MouseEvent.CLICK, customClearBtnClickHandler);
			addChild(btnCustomClear);
			
			// play sequence button
			btnPlaySequence = new RoundBtn(PLAY_SEQUENCE_TEXT, 12);
			btnPlaySequence.y = stage.stageHeight+2;
			btnPlaySequence.x = btnCustomClear.x -(model.gutter * 2) - btnPlaySequence.width;
			btnPlaySequence.addEventListener(MouseEvent.CLICK, playSeqBtnClickHandler);
			addChild(btnPlaySequence);
			
		}
		
		private function makePageButtons():void {

			var xPos:Number = model.aPages[0].x + model.aPages[0].width;
			for (var i:int = model.aPages.length; i > 0; i--){
				var text:String;
				//new page button
				if (i == model.aPages.length){
					text = "custom";
				} else {
					text = i.toString();
				}
				var btnPageSelector:PageNumberBtn = new PageNumberBtn(text, 10);
				btnPageSelector.id = i;
				btnPageSelector.y = tfTitle.y;
				xPos = xPos - btnPageSelector.width - (model.gutter * 2);
				btnPageSelector.x = xPos;
				model.aPgBtns.push(btnPageSelector);
				btnPageSelector.addEventListener(MouseEvent.CLICK, pageSelectorHandler);
				addChild(btnPageSelector);
			}

			model.aPgBtns.reverse();

			var pgTitle:TextField;
			pgTitle = new TextField();
			pgTitle.selectable = false;
			pgTitle.autoSize = TextFieldAutoSize.LEFT;
			pgTitle.background = false;

			var txtFmt:TextFormat = new TextFormat();
			txtFmt.font = "Verdana";
			txtFmt.size = 12;
			txtFmt.bold = true;
			txtFmt.color = 0xFFFFFF;
			//txtFmt.letterSpacing = 1;
			pgTitle.defaultTextFormat = txtFmt;
			pgTitle.setTextFormat(txtFmt);
			pgTitle.text = "pages:";
			addChild(pgTitle);
			pgTitle.x = model.aPgBtns[0].x - (model.gutter * 2) - pgTitle.width;
			pgTitle.y = tfTitle.y + tfTitle.height - pgTitle.height;


		}



		private function makeSampleButtons():void {
			// create sounds array
			// make buttons
			var soundBtn:CustomSoundButton;
			var rowCount:uint = 0;
			var xPos:Number = 0;
			var page:SoundsPage = new SoundsPage(model);
			model.aPages.push(page);
			pagesHolder.addChild(page);

			for each (var sound:XML in model.dataXML.sounds.sound){
				model.allSounds.push({file: sound.@src, label: sound.@quote, button: null});
			}
			//iterate through array, creating non-user pages
			for (var i:uint; i < model.allSounds.length; i++) {
				soundBtn = new CustomSoundButton(
													model,
													model.allSounds[i].label,
													model.allSounds[i].file
													);
				//soundBtn.id = i;
				if (page.addSound(soundBtn) == false) {
					//new page
					page = new SoundsPage(model);
					pagesHolder.addChild(page);
					//pageGutter = stage.stageWidth / 2 - model.aPages[0].width / 2;
					pageGutter = Main.APP_WIDTH / 2 - model.aPages[0].width / 2;
					page.x = (model.aPages.length * Main.APP_WIDTH) + pageGutter;
					model.aPages.push(page);
					//add sound to new page; initializes
					page.addSound(soundBtn);
				}
				//initializes sound
				model.allSounds[i].button = soundBtn;

			}
			//re-x the first page
			model.aPages[0].x = pageGutter;
			
			// make custom user page
			page = new SoundsPage(model);
			pagesHolder.addChild(page);
			page.x = (model.aPages.length * Main.APP_WIDTH) + pageGutter;
			model.aPages.push(page);
			pagesHolder.addChild(page);
			addChild(pagesHolder);
			// make alert text
			tfAlert = new TextField();
			tfAlert.selectable = false;
			tfAlert.autoSize = TextFieldAutoSize.CENTER;
			tfAlert.background = true;
			tfAlert.backgroundColor = ALERT_BG;

			var txtFmt:TextFormat = new TextFormat();
			txtFmt.font = "Verdana";
			txtFmt.size = 16;
			txtFmt.bold = true;
			txtFmt.color = 0x000000;
			txtFmt.blockIndent = 6;
			//txtFmt.letterSpacing = 1;
			tfAlert.defaultTextFormat = txtFmt;
			tfAlert.setTextFormat(txtFmt);
			tfAlert.text = NO_SOUNDS_TEXT;
			tfAlert.x = Main.APP_WIDTH/2 - tfAlert.width/2;
			tfAlert.y = Main.APP_HEIGHT / 2 -  tfAlert.height / 2;
			showAlert(false);
			addChild(tfAlert);
			//trace("SB: pages = " + model.aPages.length);
		}
		
		private function initButton(e:Event):void 
		{
			var soundBtn:CustomSoundButton = model.getAddedSound();
			// load the sound
			var req:URLRequest = new URLRequest(soundBtn.soundPath);
			var snd:Sound = new Sound();
			snd.addEventListener(IOErrorEvent.IO_ERROR, soundLoadErrorHandler);
			snd.load(req);
			soundBtn.sound = snd;
			// add event listener for button
			soundBtn.addEventListener(MouseEvent.MOUSE_DOWN, soundButtonDownHandler);
			soundBtn.addEventListener(MouseEvent.MOUSE_UP, soundButtonUpHandler);
		}

		private function soundLoadErrorHandler(e:IOErrorEvent):void {
			trace("Soundboard: Sound Load Error:\n" + e);
		}

		private function showPage(pgNumber:Number):void {
			//trace("SOUNDBOARD: showPage  =  "+pgNumber) ;
			var dest:Number;
			if (pgNumber > model.currPage){
				dest = pagesHolder.x - (Main.APP_WIDTH * (pgNumber - model.currPage));
				TweenLite.to(pagesHolder, .4, {x: dest, ease: Elastic.easeOut});
				model.currPage = pgNumber;
			}
			if (pgNumber < model.currPage){
				dest = pagesHolder.x + (Main.APP_WIDTH * (model.currPage - pgNumber));
				TweenLite.to(pagesHolder, .4, {x: dest, ease: Elastic.easeOut});
				model.currPage = pgNumber;
			}
			// check if on custom page
			if (pgNumber == model.aPages.length) {
				// on user custom page:
				btnCustom.setText(CUSTOM_DELETE_TEXT);
				if (model.aPages[model.aPages.length-1].aPageSounds.length < 1) {
					customSoundsExist(false);
				} else {
					customSoundsExist(true);
				}	
			} else {
				// not on user custom page
				btnCustom.setText(CUSTOM_ADD_TEXT);
				showAlert(false);
				showCustomSeqBtn(false);
			}

			model.currPage = pgNumber;
		}
		
		private function customDrop():void {
			//trace("custom drop " + model.selectingButton.getLabel());
			showPage(model.aPages.length);
		}

		private function pageSelectorHandler(e:MouseEvent):void {
			showPage(e.target.parent.id);
		}

		private function haltSounds():void {
			sndC.stop();
		}
		
		private function playSound(snd:Sound):void {
			try {
				sndC.stop();
				sndC = snd.play();
				sndC.addEventListener(Event.SOUND_COMPLETE, model.soundChannelState.soundCompleteHandler);

			} catch (err:Error){
				trace(err.message);
			}
		}

		private function grabbingEnterFrameHandler(e:Event):void {
			//var ghostBtn:CustomSoundButton = model.ghostBtn;
			model.ghostBtn.x = mouseX - model.ghostBtn.width / 2;
			model.ghostBtn.y = mouseY - model.ghostBtn.height / 2;

			var sb:Array = model.aPages[model.currPage - 1].aPageSounds;
			var swapping:Boolean;
			for (var i:int = 0; i < sb.length; i++){
				// compute stage x,y of button
				var xCheck:Number = sb[i].x + pageGutter;
				var yCheck:Number = sb[i].y + model.aPages[model.currPage - 1].y;
				// check if mouse dragging ghost button is over iterated page soundbutton
				if (mouseX > xCheck && mouseX < (xCheck + sb[i].width) && mouseY > yCheck && mouseY < (yCheck + sb[i].height)){
					model.swapToSelection = sb[i];
					model.swapToSelection.setSwapState(true);
					swapping = true;
				} else {
					if (model.selectingButton != sb[i]){
						sb[i].setSwapState(false);
					}
				}

				if (swapping){
					model.btnActionState = new SbStSwappingButtons(model);
				} else {
					model.btnActionState = new SbStGrabButton(model);
				}

				// check if hitting custom add, delete button
				if (model.ghostBtn.hitTestObject(btnCustom)){
					if (model.currPage == model.aPages.length){
						// on custom page, so dragging to button will delete
						model.btnActionState = new SbStCustomDelete(model);
					} else {
						// on reg page, so dragging to button will add to custom page
						model.btnActionState = new SbStCustomAdd(model);
					}
					btnCustom.setActive(false);
				} else {
					btnCustom.setActive(true);
				}

			}

		}

		private function soundButtonHoldListener(e:TimerEvent):void {
			haltTimer();
			model.btnActionState = new SbStGrabButton(model);
			makeGhostButton();
			this.addEventListener(Event.ENTER_FRAME, grabbingEnterFrameHandler);
		}

		private function makeGhostButton():void {
			var ghostButton:CustomSoundButton = new CustomSoundButton(model, model.selectingButton.getLabel(), model.selectingButton.soundPath);
			//ghostButton.soundPath = model.selectingButton.soundPath;
			ghostButton.alpha = .4;
			ghostButton.addEventListener(MouseEvent.MOUSE_UP, soundButtonUpHandler);
			this.addChild(ghostButton);
			model.ghostBtn = ghostButton;
			model.selectingButton.setSwapState(true);
		}

		private function haltTimer():void 
		{
			soundBtnHoldTimer.stop();
			soundBtnHoldTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, soundButtonHoldListener);
		}
		
		
		private function soundButtonDownHandler(e:MouseEvent):void {	
			if (e.target.parent is CustomSoundButton) {
				model.btnActionState = new SbStSelectingButton(model);
				model.selectingButton = e.target.parent;
				soundBtnHoldTimer = new Timer(0, TIMER_WAIT);
				soundBtnHoldTimer.addEventListener(TimerEvent.TIMER_COMPLETE, soundButtonHoldListener);
				soundBtnHoldTimer.start();
			}
		}

		private function soundButtonUpHandler(e:MouseEvent):void {
			haltTimer();
			model.btnActionState.soundButtonUp();
		}		
		
		private function feverBtnClickHandler(e:MouseEvent = null):void {
			
			if (model.soundChannelState is SndStFeverDreaming) {
				// stop fever dream
				stopFeverDream();
			} else {
				startFeverDream();
			}
		}		
		
		private function startFeverDream():void 
		{
				model.soundChannelState = new SndStFeverDreaming(model);
				btnFeverDream.setText(FEVER_OFF_TEXT);
				model.soundChannelState.playSound();
				model.bitmapState = new BitStFever(model);
		}
		
		private function stopFeverDream():void 
		{
				haltSounds();
				model.soundChannelState = new SoundboardSoundState(model);
				btnFeverDream.setText(FEVER_ON_TEXT);
				model.bitmapState = new BitStNormal(model);
		}
		
		private function playSeqBtnClickHandler(e:MouseEvent):void 
		{
			model.soundChannelState = new SndStCustSequence(model);
			SndStCustSequence(model.soundChannelState).getSequence();
			model.soundChannelState.playSound();
		}
		
		private function customClearBtnClickHandler(e:MouseEvent):void 
		{
			var custPage:SoundsPage = model.aPages[model.aPages.length - 1];
			while (custPage.aPageSounds.length > 0 ) {
				custPage.removeSound(custPage.aPageSounds[0]);
			}
			customSoundsExist(false);

		}
		private function customBtnOverHandler(e:MouseEvent):void {
			btnCustom.setText(CUSTOM_OVER_TEXT);
		}

		private function customBtnOutHandler(e:MouseEvent):void {
			if (model.currPage ==  model.aPages.length){
				//trace("on custom page");
				btnCustom.setText(CUSTOM_DELETE_TEXT);
			} else {
				btnCustom.setText(CUSTOM_ADD_TEXT);
			}
		}

		public function playSoundUpdate(e:Event):void {

			this.playSound(model.selectedSound);
		}

		public function swapUpdate(e:Event):void {
			this.removeChild(model.ghostBtn);
			removeEventListener(Event.ENTER_FRAME, grabbingEnterFrameHandler);
			// if last page, change the array for sequencing
			if (model.currPage == model.aPages[model.aPages.length-1] ) {
				//update array
			}
		}

		public function removeGhostButton(e:Event):void {
			this.removeChild(model.ghostBtn);
			this.removeEventListener(Event.ENTER_FRAME, grabbingEnterFrameHandler);
		}
		
		private function customDeleteButtonUpdate(e:Event):void 
		{
			var customPage:SoundsPage = model.aPages[model.aPages.length - 1];
			this.removeChild(model.ghostBtn);
			removeEventListener(Event.ENTER_FRAME, grabbingEnterFrameHandler);
			customPage.removeSound(model.selectingButton);
			if (customPage.aPageSounds.length < 1) {
				customSoundsExist(false);		
			}
			btnCustom.setActive(true);	
		}
		
		private function customSoundsExist(bool:Boolean):void 
		{
			if(bool){
				showAlert(false);
				showCustomClearBtn(true);
				showPlaySeqBtn(true);
			} else {
				tfAlert.text = NO_SOUNDS_TEXT;
				showAlert(true);
				showCustomClearBtn(false);
				showPlaySeqBtn(false);
				//tfAlert.alpha = 1;		
			}
		}
		
		private function showCustomClearBtn(show:Boolean):void 
		{
			if (show) {
				btnCustomClear.y = btnFeverDream.y;
			} else {
				btnCustomClear.y = stage.stageHeight+2;
			}
		}
		
		private function showPlaySeqBtn(show:Boolean):void 
		{
			if (show) {
				btnPlaySequence.y = btnFeverDream.y;
			} else {
				btnPlaySequence.y = stage.stageHeight+2;
			}
		}	
		private function customAddButtonUpdate(e:Event):void 
		{
			// jump to customPage
			removeEventListener(Event.ENTER_FRAME, grabbingEnterFrameHandler);
			var customPage:SoundsPage = model.aPages[model.aPages.length - 1];
			model.ghostBtn.alpha = 1;
			model.ghostBtn.removeEventListener(MouseEvent.MOUSE_UP, soundButtonUpHandler);
			//adds Sound -- returns if could add sound to page
			if (customPage.addSound(model.ghostBtn) == false) {
				//alert that the page is full
				showAlert(true);
				//tfAlert.alpha = 1;
				tfAlert.text = USER_PAGE_FULL_TEXT;
				tfAlert.addEventListener(MouseEvent.CLICK, tfAlertClickHandler);
				trace("SOUNDBOARD: user page is full!");	
				model.removeGhostBtn();
				model.selectingButton.setSwapState(false);
			}
			btnCustom.setActive(true);
		}
		
		private function tfAlertClickHandler(e:MouseEvent):void 
		{
			tfAlert.removeEventListener(MouseEvent.CLICK, tfAlertClickHandler);
			//tfAlert.alpha = 0;
			showAlert(true);
		}
		
		private function showAlert(show:Boolean):void 
		{
			(show) ?  tfAlert.y =  Main.APP_HEIGHT/2 -  tfAlert.height/2  :tfAlert.y = Main.APP_HEIGHT +5 ;
		}
		
		private function showCustomSeqBtn(showBtn:Boolean):void 
		{
			(showBtn) ? btnPlaySequence.y = btnFeverDream.y  : btnPlaySequence.y = Main.APP_HEIGHT +2;
		}
	}
}